#include <basictypes.h>
#include <ucos.h>
#include <ucosmcfc.h>
#include <constants.h>
#include "StopWatch.h"
#include <string.h>
#include <cfinter.h>
#include <sim.h>
#include <stdio.h>

#ifdef MOD5270B
#ifndef MOD5270
#define MOD5270
#endif
#endif

#ifdef MOD5213
#define FAST_USER_VAR
#define ter dter
#define tmr dtmr
#define trr dtrr
#define tcn dtcn
#endif

#if( defined MOD5270 || defined MOD5282 || defined MOD5272 || defined MOD5234 || defined MOD5213 )
#include <pins.h>
#endif

/* An interrupt setup helper defined in bsp.c */
extern "C"
{
#if( defined MCF5270 || defined MCF5213 )
   /* This function sets up the 5270 interrupt controller */
   void SetIntc(long func, int vector, int level, int prio );
#elif( defined MCF5282 || defined MCF5234 )
   void SetIntc(int intc, long func, int vector, int level, int prio );
#endif
}

volatile int PS FAST_USER_VAR;
volatile DWORD HighTimerReg[4] FAST_USER_VAR;
OS_SEM TimerDelaySem[4] FAST_USER_VAR;
DWORD FirstInit[4] FAST_USER_VAR;
volatile DWORD TimerDelayCalled[4]FAST_USER_VAR;
void ( *StopWatch_Interrupt_Function )( int Timer ) FAST_USER_VAR;

INTERRUPT(Timer0_Interrupt, 0x2700 )
{
	sim.timer[0].ter = 0x2;
	HighTimerReg[0]++;
	if( StopWatch_Interrupt_Function )
		StopWatch_Interrupt_Function(0);
	if( TimerDelayCalled[0] )
	{
		OSSemPost( &TimerDelaySem[0] );
		StopWatchStopClear( 0 );
		TimerDelayCalled[0] = FALSE;
	}
}
INTERRUPT(Timer1_Interrupt, 0x2700 )
{
#ifdef MCF5272
	sim.timer2.ter &= 0x02;	// clear Timer2 event condition
#else
	sim.timer[1].ter = 0x2;
#endif
	HighTimerReg[1]++;
	if( StopWatch_Interrupt_Function )
		StopWatch_Interrupt_Function(1);
	if( TimerDelayCalled[1] )
	{
		OSSemPost( &TimerDelaySem[1] );
		StopWatchStopClear(1);
		TimerDelayCalled[1] = FALSE;
	}
}
INTERRUPT(Timer2_Interrupt, 0x2700 )
{
#ifdef MCF5272
	sim.timer3.ter &= 0x02;	// clear Timer2 event condition
#else
	sim.timer[2].ter = 0x2;
#endif
	HighTimerReg[2]++;
	if( StopWatch_Interrupt_Function )
		StopWatch_Interrupt_Function(2);
	if( TimerDelayCalled[2] )
	{
		OSSemPost( &TimerDelaySem[2] );
		StopWatchStopClear(2);
		TimerDelayCalled[2] = FALSE;
	}
}
INTERRUPT(Timer3_Interrupt, 0x2700 )
{
#ifdef MCF5272
	sim.timer4.ter &= 0x02;	// clear Timer2 event condition
#else
	sim.timer[3].ter = 0x2;
#endif
	HighTimerReg[3]++;
	if( StopWatch_Interrupt_Function )
		StopWatch_Interrupt_Function(3);
	if( TimerDelayCalled[3] )
	{
		OSSemPost( &TimerDelaySem[3] );
		StopWatchStopClear(3);
		TimerDelayCalled[3] = FALSE;
	}
}

void StopWatchInit( int Timer, double InterruptTime, WORD Prescaler )
{
   StopWatchStopClear( Timer );
   PS = Prescaler;
   DWORD ReferenceTicks = 0xFFFFFFFF;
   if( InterruptTime != 0 )
   {
	    for( PS = Prescaler; PS < 0xFFFF; PS++ )
	    {
#ifdef MCF5272
	    	if( InterruptTime < ( (double)( PS + 1.0 ) * (double)0xFFFF ) )
#else
	    	if( InterruptTime < ( (double)( PS + 1.0 ) * (double)0xFFFFFFFF ) )
#endif
	    		break;
	    }
		InterruptTime /= (double)( PS + 1.0 );
		ReferenceTicks = (InterruptTime/(double)TIME_PER_CLK);
   }

#if defined MCF5272
   switch( Timer )
   {
	   case 1:
		   sim.timer2.tmr = (0x19 | ((WORD)Prescaler<<8) );   // Enable
		   sim.timer2.trr = (WORD)ReferenceTicks;
		   break;
	   case 2:
		   sim.timer3.tmr = (0x19 | ((WORD)Prescaler<<8) );   // Enable
		   sim.timer3.trr = (WORD)ReferenceTicks;
		   break;
	   case 0:	// Timer 0 is used for OSTick;
	   case 3:
	   default:
		   sim.timer4.tmr = (0x19 | ((WORD)Prescaler<<8) );   // Enable
		   sim.timer4.trr = (WORD)ReferenceTicks;
		   break;
   }

#else
   sim.timer[Timer].tmr = (0x19 | ((WORD)Prescaler<<8) );   // Enable
   sim.timer[Timer].trr = ReferenceTicks;
#endif

    TimerDelayCalled[Timer] = FALSE;

    if( FirstInit[Timer] == FALSE )
	{
       OSSemInit( &TimerDelaySem[Timer], 0 );
#ifdef MCF5272
	   if( Timer == 1 )
	   {
		   vector_base.table[70] = (long)&Timer1_Interrupt;
		   sim.icr1 = 0x00000E00;
}
	   else if( Timer == 2 )
{
		   vector_base.table[71] = (long)&Timer2_Interrupt;
		   sim.icr1 = 0x000000E0;
	   }
	   else
	   {
		   vector_base.table[72] = (long)&Timer3_Interrupt;
		   sim.icr1 = 0x0000000E;
	   }
#elif( defined MCF5270 || defined MCF5213 )
	   if( Timer == 0 ) SetIntc((long)&Timer0_Interrupt, 19+Timer, 6, 4+Timer );
	   else if( Timer == 1 ) SetIntc((long)&Timer1_Interrupt, 19+Timer, 6, 4+Timer );
	   else if( Timer == 2 ) SetIntc((long)&Timer2_Interrupt, 19+Timer, 6, 4+Timer );
	   else if( Timer == 3 ) SetIntc((long)&Timer3_Interrupt, 19+Timer, 6, 4+Timer );
#elif( defined MCF5282 || defined MCF5234 )
	   if( Timer == 0 ) SetIntc(0, (long)&Timer0_Interrupt, 19+Timer, 6, 4+Timer );
	   else if( Timer == 1 ) SetIntc(0, (long)&Timer1_Interrupt, 19+Timer, 6, 4+Timer );
	   else if( Timer == 2 ) SetIntc(0, (long)&Timer2_Interrupt, 19+Timer, 6, 4+Timer );
	   else if( Timer == 3 ) SetIntc(0, (long)&Timer3_Interrupt, 19+Timer, 6, 4+Timer );
#else
#error PROCESSOR NOT DEFINED
#endif
	   FirstInit[Timer] = TRUE;
	}
}

void StopWatchStart( int Timer )
{
#if defined MCF5272
	if( Timer == 1 )
		sim.timer2.tmr |= 0x2;
	else if( Timer == 2 )
		sim.timer3.tmr |= 0x2;
	else
		sim.timer4.tmr |= 0x2;

#else
   sim.timer[Timer].tmr |= 0x2;
#endif

}

DWORD StopWatchReadHigh( int Timer )
{
   return HighTimerReg[Timer];
}

DWORD ReadStopWatchLow( int Timer )
{
#if defined MCF5272
	if( Timer == 1 )
   return ( sim.timer2.tcn);
	else if( Timer == 2 )
		return ( sim.timer3.tcn);
	else
		return ( sim.timer4.tcn);
#else
   return sim.timer[Timer].tcn;
#endif
}

double StopWatchReadTime( int Timer )
{
	USER_ENTER_CRITICAL();
#ifdef MCF5272
	double CurrentTime;
	if( Timer == 1 )
		CurrentTime = ( (double)TIME_PER_CLK * ( (double)ReadStopWatchLow( Timer ) + ( (double)HighTimerReg[Timer] * ( (double)sim.timer2.trr + 1.0 ) ) ) * (double)(PS + 1.0) );
	else if( Timer == 2 )
		CurrentTime = ( (double)TIME_PER_CLK * ( (double)ReadStopWatchLow( Timer ) + ( (double)HighTimerReg[Timer] * ( (double)sim.timer3.trr + 1.0 ) ) ) * (double)(PS + 1.0) );
	else
		CurrentTime = ( (double)TIME_PER_CLK * ( (double)ReadStopWatchLow( Timer ) + ( (double)HighTimerReg[Timer] * ( (double)sim.timer4.trr + 1.0 ) ) ) * (double)(PS + 1.0) );

#else
   double CurrentTime = ((double) TIME_PER_CLK * ((double) ReadStopWatchLow(Timer) + ((double) HighTimerReg[Timer] * ((double) sim.timer[Timer].trr + 1.0))) * (double) (PS + 1.0));
#endif
	USER_EXIT_CRITICAL();
	return ( CurrentTime );
}

void StopWatchStop( int Timer )
{
#if defined MCF5272

	if( Timer == 1 )
      sim.timer2.tmr &= ~(0x2);
	else if( Timer == 2 )
      sim.timer3.tmr &= ~(0x2);
   else
      sim.timer4.tmr &= ~(0x2);

#else

   sim.timer[Timer].tmr &= ~(0x2);

#endif
}

void StopWatchStopClear(int Timer)
{
#if defined MCF5272
   if( Timer == 1 )
	{
	   sim.timer2.tmr &= ~(0x2);
      sim.timer2.tcn = 0x0;
      sim.timer2.ter = 0x3;
	}
	else if( Timer == 2 )
	{
	   sim.timer3.tmr &= ~(0x2);
	   sim.timer3.tcn = 0x0;
	   sim.timer3.ter = 0x3;
	}
	else
	{
	   sim.timer4.tmr &= ~(0x2);
	   sim.timer4.tcn = 0x0;
	   sim.timer4.ter = 0x3;
	}
#else
   sim.timer[Timer].tmr &= ~(0x2);
   sim.timer[Timer].tcn = 0x0;
   sim.timer[Timer].ter = 0x3;
#endif
   HighTimerReg[Timer] = 0;
}

void StopWatchDelay( double DelayTime, int Timer )
{
	StopWatchInit( Timer, DelayTime );
	StopWatchStart( Timer );
	TimerDelayCalled[Timer] = TRUE;
	OSSemPend( &TimerDelaySem[Timer], 0 );
	return;
}

void StopWatchPollingDelay( double DelayTime, int Timer )
{
   StopWatchStopClear( Timer );
   DWORD ReferenceTicks = (DelayTime/(double)TIME_PER_CLK);
#if defined MCF5272
	if( Timer == 1 )
	{
       sim.timer2.tmr = 0x9;
       sim.timer2.trr = ReferenceTicks;
      StopWatchStart( Timer );
       while( ( sim.timer2.ter & 0x2 ) == 0 );
    }
    else	if( Timer == 2 )
	{
       sim.timer3.tmr = 0x9;
       sim.timer3.trr = ReferenceTicks;
      StopWatchStart( Timer );
       while( ( sim.timer3.ter & 0x2 ) == 0 );
   }
    else
	{
       sim.timer4.tmr = 0x9;
       sim.timer4.trr = ReferenceTicks;
      StopWatchStart( Timer );
       while( ( sim.timer4.ter & 0x2 ) == 0 );
   }

#else
	sim.timer[Timer].tmr = 0x9;
	sim.timer[Timer].trr = ReferenceTicks;
   StopWatchStart( Timer );
   while ((sim.timer[Timer].ter & 0x2) == 0)
      ;
#endif
   
}	
   
void StopWatchClockGenerator( double Frequency, int Timer )
{
	double ToggleTime = (double)( 2.0 /  Frequency );
	StopWatchInit( Timer, ToggleTime );
#if defined MCF5272
	if( Timer == 1 )
	{
		sim.timer2.tmr &= ~(0x10);// Disable interrupts
		sim.timer2.tmr |= 0x20;
    }
#else
   sim.timer[Timer].tmr &= ~(0x10);   // Disable interrupts
   sim.timer[Timer].tmr |= 0x20;
#endif
   StopWatchStart( Timer );
}





