/*
 * This application demonstrates how to use the functions provided by the StopWatch utility
 *
 *
 */

#include "predef.h"
#include <stdio.h>
#include <ctype.h>

#ifdef MOD5213
#include <basictypes.h>
#include <serialirq.h>
#include <system.h>
#include <constants.h>
#include <ucos.h>
#include <SerialUpdate.h>
#include <smarttrap.h>
#include <gdbstub.h>

#else
#include <startnet.h>
#include <autoupdate.h>
#include <dhcpclient.h>
#include <NetworkDebug.h>
#endif

#if( ( defined MOD5270 ) || ( defined MOD5282 ) || (defined MOD5234) || (defined MOD5213)  )
#include <pins.h>
#endif

#include <utils.h>
#include <stdlib.h>
#include "StopWatch.h"
#include <taskmon.h>

extern "C"
{
void UserMain(void * pd);
}

const char * AppName="Timer_Example";

DWORD LEDValue = 0;
double TotalTime = 0;
char Buffer[1000];

void IncrementLEDs( int Timer );

void UserMain(void * pd)
{

#ifdef MOD5213
    SimpleUart(0,SystemBaud);
    assign_stdio(0);
    EnableSerialUpdate();
#else
    InitializeStack();
   if (EthernetIP == 0)
      GetDHCPAddress();
    EnableAutoUpdate();
   EnableTaskMonitor();
#endif

OSChangePrio(MAIN_PRIO);
    
    iprintf("%s application started\n", AppName);
	 
    while (1)
    {
    	iprintf("\r\n1) Use StopWatch to time an operation\r\n");
		iprintf("2) Use StopWatch to repeatedly trigger a function\r\n");
		iprintf("3) Use StopWatch to generate a precise OSTimeDly()\r\n");
		iprintf("4) Use StopWatch to generate a precise polling delay\r\n");
		iprintf("5) Use StopWatch to generate a Clock output\r\n");
		iprintf("Select the operation to test :");
		gets( Buffer );
		iprintf("\r\n");
		switch( Buffer[0] )
		{
			case '1':
			{
				StopWatchInit();
				iprintf("Press enter to start StopWatch... \r\n");
				gets( Buffer );
				StopWatchStart();
				iprintf("Press enter to stop StopWatch... \r\n");
				gets( Buffer );
				TotalTime = StopWatchReadTime();
				printf("The total time elapsed was %g seconds\r\n", TotalTime );
				StopWatchStopClear();
				break;
			}
			case '2':
			{
				StopWatch_Interrupt_Function = IncrementLEDs;  // Point timer interrupt call-back to LED function
            iprintf(
                  "Input number of seconds (floating point) between LED increments:");
				gets( Buffer );
				StopWatchInit( 1, strtod( Buffer, NULL ) );
				StopWatchStart();
				iprintf("\r\nPress enter to disable incrementing LEDs\r\n");
				gets( Buffer );
				StopWatchStopClear();
				StopWatch_Interrupt_Function = NULL;
				LEDValue = 0;
				putleds( 0 );
				break;
			}
			case '3':
			{
            iprintf(
                  "Input number of seconds (floating point) for precise OSTimeDly():");
				gets( Buffer );
				DWORD StartingTick = TimeTick;
				StopWatchDelay( strtod( Buffer, NULL ) );
            iprintf("\r\nDelay finished in %d system ticks\r\n", (TimeTick
                  - StartingTick));
				break;
			}
			case '4':
    {
            iprintf(
                  "Input number of seconds (floating point) for precise polling time delay:");
            gets( Buffer );
            double DelayTime = strtod( Buffer, NULL );
            iprintf(
                  "\r\nInput number of consecutive polls to perform for this test:");
            gets( Buffer );
            int DelayRepititions = strtol( Buffer, NULL, 10 );
            DWORD StartingTick = TimeTick;
            for( int i = 0; i < DelayRepititions; i++ )
               StopWatchPollingDelay( DelayTime );
            iprintf("\r\nDelay finished in %d system ticks\r\n", (TimeTick
                  - StartingTick));
            break;
			}
			case '5':
    {
//--------------------------------------------------
// NOTE - You must manually set the correct timer output pin for the
//        proper timer output function for the clock generator to work
//--------------------------------------------------
#if( ( defined MOD5270 ) || ( defined MOD5282 ) )
				J2[34].function( PINJ2_34_DTOUT1 );
#elif(defined MOD5234)
				J2[34].function( PINJ2_34_DT1OUT );
#elif(defined MOD5213)
				Pins[23].function( PIN23_DTOUT1 );
#endif
            iprintf(
                  "Input the Frequency (floating point) to generate a clock on timer 1 out:");
				gets( Buffer );
				StopWatchClockGenerator( strtod( Buffer, NULL ) );
				iprintf("\r\n");
				break;
			}
			default:
				break;
		}
    }
}

// A simple funtion that increments the LEDs displayed on the dev-board
void IncrementLEDs( int Timer )
{
// The Timer parameter is not used here since we do not need to determine which timer interrupted
	LEDValue++;
	putleds( LEDValue );
}



